# sb::App::Install
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App::Install;
use strict;
use sb::Interface ();
use sb::Config ();
use sb::Language ();
use sb::Data ();
use sb::Template ();
use sb::Driver ();
use sb::App::Upgrade;
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.15';
# 0.15 [2008/12/16] modified _step_installing to create admin user and weblog object poperly
# 0.14 [2008/11/09] removed class_list, use sb::Data::classes instead
# 0.13 [2008/08/30] modified _move_files to fix terrible bugs
# 0.12 [2008/07/21] modified _move_files to consider the file extensions
# 0.11 [2008/07/20] added _step_versionup / modified run and common_parts to consider versionup
# 0.10 [2008/07/16] changed _check_environment to use sb::Net
# 0.09 [2007/12/04] temporary implementation
# 0.08 [2007/11/17] refactored for Serene Bach 3.*
# 0.07 [2006/08/01] changed CHECK_URL
# 0.06 [2005/07/27] changed _convert_main to convert order of links/category/user correctly
# 0.05 [2005/07/26] changed _convert_main to convert recommendation order correctly
# 0.04 [2005/07/25] changed _convert_main to convert recommendation correctly
# 0.03 [2005/07/24] changed run to check whether xml request is acceptable or not
# 0.02 [2005/07/20] changed _check_environment to check file permissions as well
# 0.01 [2005/07/19] changed _convert_main to convert entry individually
# 0.00 [2005/07/11] generated
# [note] ===========================================
# Calling sb::App::Install, sb::Driver isn't loaded.
# So it's need to load sb::Driver on your own.
# ==================================================
# --------------------------------------
# // defines
sub FILES_AT_ONCE (){ 5 };
sub BACKUP        (){ 'sb2/' };
sub CONFIG        (){ 'configure' };
sub CATEGORY_NANE (){ '%Main%->%Sub%' };
sub TEMPLATE      (){ 'install.html' };
sub CSS_NAME      (){ 'style.css' };
sub TEMP_INSTALL  (){ 'install.tmp' };
sub DUMMY_TEXT    (){ 'dummy.txt' };
sub CHECK_URL     (){ 'http://serennz.sakura.ne.jp/' };
sub DEFAULT_TITLE (){ 'My first weblog' };
sub DEFAULT_TEMP  (){ 'summer_breeze.cgi' };
# --------------------------------------
# // public functions - static values
sub library_list
{
	return (
		'required' => ['LWP::UserAgent','HTTP::Request','Jcode',],
		'optional' => ['Image::Magick','Net::SMTP','Net::POP3',],
	);
}
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'step'   => 0,
		'mode'   => '',
		'xml'    => undef,
		'has_sb' => undef,
		'temp'   => undef,
		'err'    => undef,
	);
	$self->SUPER::initialize(%param,@_);
	$self->generate_accessor( keys(%param) );
	$self->step(int($self->cgi->value('step')));
	umask(0); # set file mask
	return $self;
}
sub run
{
	my $class = shift;
	my $self = $class->new(@_);
	eval {
		$self->temp(sb::Template->new($self->load_template('file'=>TEMPLATE)));
		die(sb->string('Failed to load the template.')) if (!$self->temp);
		my $status = $self->_is_finished();
		die(sprintf(sb->string('%s has been already installled.'),$sb::PRODUCT) . "\n") if ($status eq 'done');
		if ( $status eq 'need_upgrade' )
		{ # need to update database
			$self->mode('versionup');
			$self->_step_versionup( ( $self->cgi->value('versionup') eq '' ) ? 'confirm' : 'run' );
		}
		elsif ( ($self->cgi->value('install') eq '' and $self->cgi->value('upgrade') eq '')
		     or $self->step() == 0 )
		{ # the first step
			$self->_check_environment();
		}
		elsif ($self->cgi->value('install') ne '')
		{ # installing
			$self->mode('install');
			$self->_step_installing();
		}
		elsif ($self->cgi->value('upgrade') ne '')
		{ # upgrading
			$self->mode('upgrade');
			$self->_step_upgrading();
		}
	};
	if ($@)
	{
		$self->error($@);
	}
	else
	{
		$self->common_parts($self->temp);
		$self->output($self->temp->output);
	}
}
# --------------------------------------
# // override base class - template parts
sub common_parts
{
	my $self = shift;
	my $temp = shift;
	return if (!$temp or !$self);
	$self->SUPER::common_parts($temp);
	$temp->tag('sb_install_cgi'=>$self->conf->value('basic_install'));
	$temp->tag('sb_site_cgi'=>$self->conf->value('conf_srv_cgi') . $self->conf->value('basic_admn'));
	$temp->tag('sb_install_mode'=>$self->mode());
	if ( $self->mode ne 'versionup' )
	{
		$temp->tag('sb_step' . $self->step() . '_on'=>' class="current"');
		$temp->block('sb_upgrade_menu'=>1) if ($self->mode() eq 'upgrade');
		$temp->block('sb_install_menu'=>1) if ($self->mode() ne 'upgrade');
		$temp->block('sb_install_step' . $self->step() => 1);
		if ( $self->err() )
		{
			$temp->tag('sb_error_message'=>$self->err());
			$temp->block('sb_install_error'=>1);
		}
	}
}
sub resource_dir
{
	my $self = shift;
	return $self->conf->value('dir_temp') . $self->lang->language . '/';
}
sub static_path
{
	my $self = shift;
	return $self->conf->value('srv_temp');
}
sub style_path
{
	my $self = shift;
	return $self->static_path . CSS_NAME;
}
# --------------------------------------
# // private functions - common
sub _is_finished
{
	my $self = shift;
	my $blog = undef;
	eval
	{
		$self->_set_driver();
		$blog = sb::Data->load('Weblog','id'=>0);
	};
	return 'not_yet' if ( !$blog or $@ );
	return 'need_upgrade' if ( !$blog->dbset or int($blog->dbset) < $sb::DBTABLE );
	return 'working' if ( $self->_check_temporary(TEMP_INSTALL) );
	return 'done';
}
sub _has_previous_ver
{
	my $self = shift;
	my $dir  = $self->conf->value('dir_data');
	$dir = $dir . BACKUP if (-e $dir . BACKUP);
	my $ext = $self->conf->value('file_suf');
	return 1 if (-e $dir . 'id' . $ext and -e $dir . 'template/');
	return;
}
sub _check_environment
{
	my $self = shift;
	my $flag = undef;
	# === checking external libralies ===
	my %list_of = $self->library_list();
	foreach my $check ('required','optional')
	{
		my $cnt = 0;
		my @libs = @{$list_of{$check}};
		foreach my $lib (@libs)
		{
			next if ($self->lang->language ne 'ja' and $lib eq 'Jcode');
			my $text = '';
			eval("require $lib;");
			$text = ($@) 
				? '<strong style="color:red">' . sb->string('%s is NOT installed.') . '</strong>'
				: '<strong style="color:green">' . sb->string('%s is installed.') . '</strong>';
			$flag = 1 if ($@ and $check eq 'required');
			$self->temp->num($cnt);
			$self->temp->tag('sb_check_lib_status_' . $check=>sprintf($text,$lib));
			$self->temp->tag('sb_check_list_class'=>($cnt % 2) ? 'odd' : 'even');
			$cnt++;
		}
		$self->temp->block('check_lib_' . $check => $cnt);
	}
	# === checking outer access ===
	eval {
		die(sb->string('999 Skipped') . "\n") if ($flag);
		require sb::Net;
		my $ua = sb::Net->new()->agent();
		die($ua->request(HTTP::Request->new(GET => CHECK_URL))->status_line . "\n");
	};
	$self->temp->num(0);
	$self->temp->tag('sb_check_connect'=>$@);
	# === checking directory permissions ===
	my @dirs = ();
	push(@dirs,$self->conf->value('dir_data'));
	push(@dirs,$self->conf->value('dir_lock'));
	push(@dirs,$self->conf->value('conf_dir_base') . $self->conf->value('dir_style'));
	push(@dirs,$self->conf->value('conf_dir_base') . $self->conf->value('conf_dir_log'));
	push(@dirs,$self->conf->value('conf_dir_base') . $self->conf->value('conf_dir_img'));
	for (my $i=0;$i<@dirs;$i++)
	{
		$self->temp->num($i);
		my $text = '<strong style="color:green">' . sb->string('&quot;%s&quot; is fine.') . '</strong>';
		if (!-d $dirs[$i])
		{
			$flag = 1;
			$text = '<strong style="color:red">' . sb->string('&quot;%s&quot; does NOT exist.') . '</strong>';
		}
		elsif (!-r $dirs[$i])
		{
			$flag = 1;
			$text = '<strong style="color:red">' . sb->string('&quot;%s&quot; is NOT readable.  Please check the permission.') . '</strong>';
		}
		elsif (!-w $dirs[$i])
		{
			$flag = 1;
			$text = '<strong style="color:red">' . sb->string('&quot;%s&quot; is NOT writable.  Please check the permission.') . '</strong>';
		}
		$self->temp->tag('sb_check_dir_status'=>sprintf($text,$dirs[$i]));
		$self->temp->tag('sb_check_list_class'=>($i % 2) ? 'odd' : 'even');
	}
	$self->temp->block('check_dir'=>$#dirs + 1);
	# === checking file permission
	my @files = ();
	push(@files,$self->conf->value('conf_dir_base') . $self->conf->value('file_index'));
	push(@files,$self->conf->value('conf_dir_base') . $self->conf->value('file_css'));
	for (my $i=0;$i<@files;$i++)
	{
		$self->temp->num($i);
		my $text = '<strong style="color:green">' . sb->string('&quot;%s&quot; is fine.') . '</strong>';
		if (!-e $files[$i])
		{
			$flag = 1;
			$text = '<strong style="color:red">' . sb->string('&quot;%s&quot; does NOT exist.') . '</strong>';
		}
		elsif (!-r $files[$i])
		{
			$flag = 1;
			$text = '<strong style="color:red">' . sb->string('&quot;%s&quot; is NOT readable.  Please check the permission.') . '</strong>';
		}
		elsif (!-w $files[$i])
		{
			$flag = 1;
			$text = '<strong style="color:red">' . sb->string('&quot;%s&quot; is NOT writable.  Please check the permission.') . '</strong>';
		}
		$self->temp->tag('sb_check_file_status'=>sprintf($text,$files[$i]));
		$self->temp->tag('sb_check_list_class'=>($i % 2) ? 'odd' : 'even');
	}
	$self->temp->block('check_file'=>$#files + 1);
	# === finishing check ===
	$self->temp->num(0);
	$self->temp->tag('disable_upgrade'=>'disabled="disabled"') if (!$self->_has_previous_ver());
	$self->temp->block('check_step0_next'=>1) if (!$flag);
	$self->temp->block('check_step0_error'=>1) if ($flag);
}
# --------------------------------------
# // private functions - file manipulations
sub _create_temporary
{
	my $self = shift;
	my $file = shift;
	$file = $self->conf->value('dir_data') . $file;
	if (!-e $file)
	{
		open(INDEX,">$file");
		close(INDEX);
		chmod($self->conf->value('basic_file_attr'),$file);
	}
	die(sb->string('Failed to create a file') . $file) if (!-e $file);
}
sub _remove_temporary
{
	my $self = shift;
	my $file = shift;
	$file = $self->conf->value('dir_data') . $file;
	unlink($file);
}
sub _check_temporary
{
	my $self = shift;
	my $file = shift;
	$file = $self->conf->value('dir_data') . $file;
	return ( -e $file );
}
sub _create_config_file
{
	my $self = shift;
	my $file = $self->conf->config_file();
	if (!-e $file)
	{
		open(CONFIG,">$file");
		close(CONFIG);
		chmod($self->conf->value('basic_file_attr'),$file);
	}
	die(sb->string('Failed to create a file') . $file) if (!-e $file);
}
sub _create_global_lock
{
	my $self = shift;
	my $lock = $self->conf->value('dir_lock') . $self->conf->value('file_lock');
	if (!-e $lock)
	{
		open(INDEX,">$lock");
		close(INDEX);
		chmod($self->conf->value('basic_file_attr'),$lock);
	}
	die(sb->string('Failed to create a file') . $lock) if (!-e $lock);
}
sub _initialize_access_data
{
	my $self = shift;
	my $file = $self->conf->value('dir_data') . $self->conf->value('file_access');
	my $dir  = $self->conf->value('dir_data') . $self->conf->value('dir_access');
	if (!-e $file)
	{
		open(INDEX,">$file");
		close(INDEX);
		chmod($self->conf->value('basic_file_attr'),$file);
	}
	die(sb->string('Failed to create a file') . $file) if (!-e $file);
	mkdir($dir,$self->conf->value('basic_dir_attr')) if (!-e $dir);
	die(sprintf(sb->string('&quot;%s&quot; is NOT readable.'),$dir)) if (!-r $dir);
	die(sprintf(sb->string('&quot;%s&quot; is NOT writable.'),$dir)) if (!-w $dir);
	my $lock = $self->conf->value('dir_lock') . $self->conf->value('file_lckcnt');
	if (!-e $lock)
	{
		open(INDEX,">$lock");
		close(INDEX);
		chmod($self->conf->value('basic_file_attr'),$lock);
	}
	die(sb->string('Failed to create a file') . $lock) if (!-e $lock);
}
# --------------------------------------
# // private functions - db manipulations
sub _create_tables
{
	my $self = shift;
	my $driver = sb->get->driver();
	die(sb->string('Failed to load database, please check database settings.')) if (!$driver);
	foreach my $name ( sb::Data->classes() )
	{
		my $class = 'sb::Data::' . $name;
		sb::load_module($class);
		$driver->create_table($class);
	}
}
sub _set_driver
{
	my $self = shift;
	eval {
		my $driver = sb::Driver->new(
			$self->conf->value('driver'),
			%{$self->conf->value('db_param')}
		);
		sb->get->driver($driver) if ($driver);
	};
	die(sb->string('Failed to load database, please check database settings.') . $@) if ($@);
}
sub _insert_data
{
	my $self = shift;
	my $obj  = shift;
	my $driver = sb->get->driver();
	die(sb->string('Failed to load database, please check database settings.')) if (!$driver);
	$driver->insert($obj);
}
# --------------------------------------
# // private functions - installing
sub _step_installing
{
	my $self = shift;
	$self->_set_driver();
	$self->_check_step_for_install();
	if ($self->step() == 1)
	{ # step 1 [ creating administrator ]
		$self->_create_temporary(TEMP_INSTALL);
		$self->_create_tables();
		$self->_create_global_lock();
		$self->temp->block('install_step1'=>1);
		$self->temp->block('install_step1_form'=>1);
		$self->temp->block('step1_error'=>1) if ($self->err() ne '');
	}
	elsif ($self->step() == 2)
	{ # step 2 [ server & directory settings ]
		my $administrator = sb::Data->load('User','id'=>0);
		if ( !$administrator )
		{
			$administrator = sb::Data->add('User',
				'name' => $self->cgi->value('__user'),
				'stat' => 0,
			);
			$administrator = undef if ($self->cgi->value('__pass') eq '');
			if ( $administrator )
			{
				$administrator->pass($self->cgi->value('__pass'));
				sb::Data->update($administrator);
			}
		}
		die(sb->string('Failed to create an administrator')) if (!$administrator);
		$self->temp->num(0);
		if ($self->{'err'} eq '')
		{
			$self->temp->tag('srv_cgi' =>$self->_get_current_uri());
			$self->temp->tag('srv_base'=>$self->_get_current_uri());
			$self->temp->tag('dir_base'=>$self->conf->value('conf_dir_base'));
			$self->temp->tag('dir_log' =>$self->conf->value('conf_dir_log'));
			$self->temp->tag('dir_img' =>$self->conf->value('conf_dir_img'));
		}
		else
		{
			my $cgi = $self->{'cgi'};
			$self->temp->tag('srv_cgi' =>$self->_check_dir($self->cgi->value('srv_cgi')));
			$self->temp->tag('srv_base'=>$self->_check_dir($self->cgi->value('srv_base')));
			$self->temp->tag('dir_base'=>$self->_check_dir($self->cgi->value('dir_base')));
			$self->temp->tag('dir_log' =>$self->_check_dir($self->cgi->value('dir_log')));
			$self->temp->tag('dir_img' =>$self->_check_dir($self->cgi->value('dir_img')));
		}
		$self->temp->block('install_step2'=>1);
		$self->temp->block('install_step2_form'=>1);
		$self->temp->block('step2_error'=>1) if ($self->err() ne '');
	}
	elsif ($self->step() == 3)
	{ # step 3 [ weblog configuration ]
		$self->_create_config_file();
		$self->conf->value('conf_srv_cgi' =>$self->_check_dir($self->cgi->value('srv_cgi')));
		$self->conf->value('conf_srv_base'=>$self->_check_dir($self->cgi->value('srv_base')));
		$self->conf->value('conf_dir_base'=>$self->_check_dir($self->cgi->value('dir_base')));
		$self->conf->value('conf_dir_log' =>$self->_check_dir($self->cgi->value('dir_log')));
		$self->conf->value('conf_dir_img' =>$self->_check_dir($self->cgi->value('dir_img')));
		$self->conf->save($self->conf->config_file());
		$self->temp->tag('blog_title'=>DEFAULT_TITLE);
		$self->temp->block('install_step3'=>1);
		$self->temp->block('install_step3_form'=>1);
		$self->temp->block('install_step3_text'=>1);
	}
	else
	{ # finishing ...
		my $blog = sb::Data->load('Weblog','id'=>0);
		$blog = sb::Data->add('Weblog') if (!$blog);
		if ( $blog )
		{
			$blog->title(sb::Text->new($self->cgi->value('blog_title'))->entitized());
			$blog->text(sb::Text->new($self->cgi->value('blog_desc'))->entitized());
			$blog->dbset($sb::DBTABLE);
			sb::Data->update($blog);
		}
		require sb::Admin::Template;
		my $temp = sb::Admin::Template::import_template($self,
			'cont' => $self->_load_template_package,
			'time' => $self->time->data,
		);
		if ($temp)
		{
			$temp->use(1);
			sb::Data->update($temp);
		}
		$self->_initialize_access_data();
		$self->_remove_temporary(TEMP_INSTALL);
	}
}
sub _check_step_for_install
{
	my $self = shift;
	die(sb->string('Unknown error.')) if ($self->step() > 1 and !$self->_check_temporary(TEMP_INSTALL));
	if ($self->step() == 2)
	{
		require sb::Admin::User;
		my $error = sb::Admin::User->check_user(
			'pass' => $self->cgi->value('__pass'),
			'conf' => $self->cgi->value('__conf'),
			'check_name' => 0,
		);
		if ($self->cgi->value('__user') =~ /^[a-zA-Z0-9_\-\.]+$/)
		{
			$self->temp->num(0);
			$self->temp->tag('install_user'=>$self->cgi->value('__user'))
		}
		else
		{
			$error = sb->string('Invalid characters in user name.');
		}
		if ($error ne '')
		{
			$self->err($error);
			$self->step(1);
		}
	}
	elsif ($self->step() == 3)
	{
		my $base = $self->_check_dir($self->cgi->value('dir_base'));
		my @dirs = ();
		push(@dirs,$base . $self->_check_dir($self->cgi->value('dir_log')));
		push(@dirs,$base . $self->_check_dir($self->cgi->value('dir_img')));
		foreach my $dir (@dirs)
		{
			$self->err('not_fine') if (!-d $dir or !-r $dir or !-w $dir);
		}
		$self->step(2) if ($self->err() ne '');
	}
}
sub _check_dir
{
	my $self = shift;
	my $dir  = shift;
	$dir .= '/' if ($dir !~ /\/$/);
	return $dir;
}
sub _get_current_uri
{
	my $self = shift;
	my $host = $ENV{'HTTP_HOST'} . $ENV{'SCRIPT_NAME'};
	$host =~ s/(.*[:\/\\])(.*)/$1/;
	return 'http://' . $host;
}
sub _load_template_package
{
	my $self = shift;
	my $file = $self->conf->value('dir_temp') . DEFAULT_TEMP;
	open(TEMPIN,"<$file") or die("Fail to open : $file");
	binmode(TEMPIN);
	my @template = <TEMPIN>;
	close(TEMPIN);
	return join('',@template);
}
sub _check_filename
{
	require sb::Admin::Template;
	return sb::Admin::Template::_check_filename(@_);
}
# --------------------------------------
# // private functions - upgrading
sub _step_upgrading
{
	my $self = shift;
	$self->_check_step_for_upgrade();
	if ($self->step() == 1)
	{ # step 1 [ login ]
		$self->_create_temporary(TEMP_INSTALL);
		$self->temp->block('upgrade_step1'=>1);
		$self->temp->block('step1_error'=>1) if ($self->err() ne '');
		return;
	}
	$self->_set_driver();
	if ($self->step() == 2)
	{ # step 2 [ backup ]
		$self->_move_files();
		$self->_create_tables();
		$self->_create_global_lock();
		$self->_create_config_file(); # create new config file
		$self->conf->save($self->conf->config_file());
		$self->temp->block('upgrade_step2'=>1);
	}
	elsif ($self->step() == 3)
	{ # step 3 [ converting ]
		my $db_new = sb->get->driver();
		my $db_old = sb::Driver->new('TextOld',
			'dir'  => $self->conf->value('dir_data') . BACKUP,
			'suf'  => $self->conf->value('file_suf'),
			'idx'  => $self->conf->value('dbtxt_ids'),
			'mod'  => $self->conf->value('basic_file_attr'),
			'lock' => $self->conf->value('dbtxt_save'),
			'code' => $self->conf->value('sb2_charcode'),
		);
		die(sb->string('Failed to load database, please check database settings.')) if (!$db_old or !$db_new);
		sb->get->driver($db_old);
		my %entry   = sb::Data->load_as_hash('Entry');
		my @classes = sb::Data->classes();
		my $step    = $self->_progress_for_step3();
		my $name    = $classes[$step];
		my @objs    = map { $self->_converter($_,\%entry) } sb::Data->load($name);
		$db_new->{'max'} = $db_old->{'max'}; # copy id table
		if ( ref($db_new) eq 'sb::Driver::Text' 
		  or ref($db_new) eq 'sb::Driver::TextOld' )
		{
			$db_new->save(@objs);
		}
		else
		{
			foreach my $obj (@objs)
			{
				$db_new->insert($obj);
			}
			$db_new->save_ids();
		}
		$step++;
		$self->_progress_for_step3($step);
		$self->temp->num(0);
		$self->temp->tag('convert_progress'=>'<strong>' . $step . '/' . ($#classes + 1)  . '</strong>');
		$self->temp->block('upgrade_step3'=>1);
		$self->temp->block(($step > $#classes) ? 'upgrade_step3_finish' : 'upgrade_step3_continue'=>1);
		$self->temp->block(($step > $#classes) ? 'upgrade_step3_finish_text' : 'upgrade_step3_continue_text'=>1);
	}
	else
	{ # finishing ...
		$self->_initialize_access_data();
		$self->_remove_temporary(TEMP_INSTALL);
	}
}
sub _converter
{
	my $self = shift;
	my $obj = shift;
	my $entry = shift;
	my $class = ref($obj);
	my %table = $obj->table();
	foreach my $elem ( $obj->elements() )
	{
		$obj->{$elem} = -1 if ($table{$elem} eq 'id' and $obj->{$elem} eq '');
		if ($class eq 'sb::Data::Message' or $class eq 'sb::Data::Trackback')
		{ # only affect message or trackback
			if ($elem eq 'stat' and $obj->{$elem} == 1)
			{ # for opened message or trackback
				if (!$entry->{$obj->{'eid'}} # entry does not exist
				 or $entry->{$obj->{'eid'}}->stat == 0 # entry is not opened
				 or ($class eq 'sb::Data::Message' and $entry->{$obj->{'eid'}}->{'acm'} == 0) # any comment is not accepted
				 or ($class eq 'sb::Data::Trackback' and $entry->{$obj->{'eid'}}->{'atb'} == 0) # any trackback is not accepted
				)
				{
					$obj->{$elem} = 2;
				}
			}
		}
		elsif ($class eq 'sb::Data::User')
		{
			$obj->{'cat'} = -1 if ($obj->{'cat'} eq '');
		}
		elsif ($class eq 'sb::Data::Category')
		{
			$obj->{'dir'} = '' if ($obj->{'dir'} eq $self->conf->value('conf_dir_log'));
		}
		elsif ($class eq 'sb::Data::Weblog')
		{
			$obj->{'dbset'} = $sb::DBTABLE;
		}
	}
	return $obj;
}
sub _progress_for_step3
{
	my $self = shift;
	my $update = shift;
	my $file = $self->conf->value('dir_data') . TEMP_INSTALL;
	if ($update)
	{
		open(TEMP,">$file") or die(sb->string('Failed to open a file.'));
		binmode(TEMP);
		print TEMP $update,"\n";
		close(TEMP);
		return;
	}
	else
	{
		open(TEMP,"<$file") or die(sb->string('Failed to open a file.'));
		my $cnt = <TEMP>;
		close(TEMP);
		return int($cnt);
	}
}
sub _check_step_for_upgrade
{
	my $self = shift;
	die(sb->string('Unknown error.')) if ($self->step() > 1 and !$self->_check_temporary(TEMP_INSTALL));
	if ($self->{'step'} == 2)
	{
		my $dir = $self->conf->value('dir_data') . BACKUP;
		$dir = $self->conf->value('dir_data') if (!-e $dir);
		my $db_old = sb::Driver->new('TextOld',
			'dir'  => $dir,
			'suf'  => $self->conf->value('file_suf'),
			'idx'  => $self->conf->value('dbtxt_ids'),
			'mod'  => $self->conf->value('basic_file_attr'),
			'lock' => $self->conf->value('dbtxt_save'),
			'code' => $self->conf->value('sb2_charcode'),
		);
		die(sb->string('Failed to load database, please check database settings.')) if (!$db_old);
		sb->get->driver($db_old);
		my $user = sb::Data->load('User','id'=>0);
		die(sb->string('Failed to load administrator.')) if (!$user);
		$self->err(sb->string('No such user.')) if ($user->name ne $self->cgi->value('__user'));
		$self->err(sb->string('Incorrect password.')) if (!$user->check_pass($self->cgi->value('__pass')));
		$self->step(1) if ($self->err() ne '');
	}
}
sub _sb2_dbfiles 
{
	return (
		'amazon',
		'category',
		'configure',
		'entry',
		'id',
		'image',
		'link',
		'message',
		'plugin',
		'session',
		'template',
		'trackback',
		'user',
		'weblog'
	);
}
sub _sb2_dbdirs
{
	return (
		'entry',
		'message',
		'template',
		'trackback',
		'user',
	);
}
sub _move_files
{
	my $self = shift;
	my $dir  = $self->conf->value('dir_data');
	my $back = $dir . BACKUP;
	my $ext = $self->conf->value('file_suf');
	mkdir($back,$self->conf->value('basic_dir_attr')) if (!-e $back);
	die(sprintf(sb->string('&quot;%s&quot; is NOT readable.'),$back)) if (!-r $back);
	die(sprintf(sb->string('&quot;%s&quot; is NOT writable.'),$back)) if (!-w $back);
	my @files = map { $_ . $ext } ( $self->_sb2_dbfiles() );
	push( @files, $self->_sb2_dbdirs() );
	push( @files, $self->conf->value('file_access') );
	push( @files, $self->conf->value('dir_access') );
	foreach my $file ( @files )
	{
		next if ( !-e $dir . $file );
		next if ( -e $back . $file ); # already backuped, so skip it
		rename($dir . $file, $back . $file);
	}
}
# --------------------------------------
# // private functions - versionup
sub _step_versionup
{
	my $self = shift;
	my $step = shift;
	if ( $step eq 'run' )
	{
		$self->_set_driver();
		my $blog = sb::Data->load('Weblog','id'=>0);
		my $executor = sb::App::Upgrade->new(
			'driver'  => sb->get->driver(),
			'current' => int( $blog->dbset ),
		);
		$executor->upgrade();
		$blog->dbset($sb::DBTABLE);
		sb::Data->update($blog);
	}
	if ( $self->err() ne '' )
	{
		$self->temp->num(0);
		$self->temp->tag('versionup_error'=>sb::Text->new($self->err())->entitized());
		$self->temp->block('versionup_error'=>1);
	}
	else
	{
		if ($step eq 'confirm')
		{
			$self->_create_temporary(TEMP_INSTALL);
			$self->temp->block('versionup_step1'=>1);
		}
		else
		{
			$self->_remove_temporary(TEMP_INSTALL);
			$self->temp->block('versionup_step2'=>1);
		}
	}
}
1;
__END__
