# sb::Build
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Build;
use strict;
use sb::Language ();
use sb::Config ();
use sb::Data ();
use sb::Content ();
use sb::Content::Feed ();
use sb::Template;
use sb::App ();
use sb::App::Feed ();
use sb::Object ();
use base qw( sb::Object );
use vars qw( $VERSION );
$VERSION = '0.14';
# 0.14 [2009/01/14] modified build_category_index to set correct category id for checking template
# 0.13 [2007/11/30] modified initialize to use sb instaed of different classes
# 0.12 [2007/09/20] changed _check_template to use "template" method instead of $self->{'template'}
# 0.11 [2007/08/17] removed some setter functions no longer used
# 0.10 [2007/08/17] changed set_callback_for_feed and build_feedfile to use sb::Content::Feed
# 0.09 [2007/08/12] modified for Serene Bach 3.*
# 0.08 [2007/05/11] changed build_category_index and build_monthly_archive
# 0.07 [2006/02/03] changed build_feedfile
# 0.06 [2005/10/18] changed _initialize to load templates as instances of TemplateManager
# 0.05 [2005/08/03] changed build_top_page to sort entries correctly
# 0.04 [2005/08/02] changed set_latest_entries to get entries for feeds
# 0.03 [2005/07/16] changed build_javascript, _script_tag, _generate_javascript to handle list type block correctly
# 0.02 [2005/07/11] changed config variable "srv_cgi" to "conf_srv_cgi"
# 0.01 [2005/07/08] added build_cookie_js
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines / static variables
sub SCRIPT_TAG         (){ '<script type="text/javascript" charset="%s" src="%s"></script>' };
sub SUBCATEGORY_PREFIX (){ 'cat' };
sub LIST_TEMPLATE      (){ "<!-- BEGIN %s -->\n{%s}\n<!-- END %s -->\n" };
sub COOKIE_JS_TEMP     (){ 'default_script.js' };
my %mListScripts = (
	'link_list'             => {'js'=>'link.js',   'block'=>'link',            },
	'user_list'             => {'js'=>'user.js',   'block'=>'profile',         },
	'recent_comment_list'   => {'js'=>'comment.js','block'=>'recent_comment',  },
	'recent_trackback_list' => {'js'=>'tb.js',     'block'=>'recent_trackback',},
	'latest_entry_list'     => {'js'=>'entry.js',  'block'=>'latest_entry',    },
	'category_list'         => {'js'=>'cat.js',    'block'=>'category',        },
	'archives_list'         => {'js'=>'arc.js',    'block'=>'archives',        },
	'calendar'              => {'js'=>'cal1.js',   'block'=>'calendar',        },
	'calendar2'             => {'js'=>'cal2.js',   'block'=>'calendar',        },
	'calendar_vertical'     => {'js'=>'calv.js',   'block'=>'calendar',        },
	'calendar_horizontal'   => {'js'=>'calh.js',   'block'=>'calendar',        },
);
my %mFeedFile = (
	'rss'  => 'index.rdf',
	'atom' => 'atom.xml',
);
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'time'         => undef, # [NUM.]current time
		'blog'         => undef, # [OBJ.]weblog data
		'user'         => undef, # [HASH]user data
		'cat'          => undef, # [HASH]category data
		'sortedcat'    => undef, # [ARRY]sorted category data
		'entry'        => undef, # [ARRY]the latest entries
		'entry_num'    => undef, # [NUM.]number of entries
		'entryinfo'    => undef, # [HASH]entry information
		'template'     => undef, # [HASH]template data
		'default_id'   => undef, # [NUM.]default template id
		'conf' => sb->get->conf,
		'lang' => sb->get->lang,
	);
	$self->SUPER::initialize(%param,@_);
	$self->generate_accessor(keys %param);
	return $self;
}
# --------------------------------------
# // public functions - executor
sub build_css
{
	my $self = shift;
	my $css  = shift; # an instance of sb::Template for css
	my $file = shift; # output file path if necessary
	$css  = $self->template()->{'css'} if (!$css);
	$file = $self->conf->value('conf_dir_base') . $self->conf->value('file_css') if ($file eq '');
	eval {
		open(CSSOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(CSSOUT);
		print CSSOUT sb::Content->output($css,'mode'=>'css');
		close(CSSOUT);
	};
	return $@ if ($@);
	return;
}
sub build_entry
{
	my $self = shift;
	my $entry = shift;
	my %param = (
		'prev' => undef,
		'next' => undef,
		@_
	);
	return sb->string('No article.') if (!$entry);
	my $content = sb::Content->new(
		'main' => {
			'archives'         => sub { _script_tag('archives_list',$_[0]); },
			'category'         => \&_category_list,
			'link'             => sub { _script_tag('link_list',$_[0]); },
			'recent_comment'   => sub { _script_tag('recent_comment_list',$_[0]); },
			'recent_trackback' => sub { _script_tag('recent_trackback_list',$_[0]); },
			'latest_entry'     => sub { _script_tag('latest_entry_list',$_[0]); },
			'profile'          => sub { _script_tag('user_list',$_[0]); },
			'calendar'         => \&_calendar_for_entry,
		}
	);
	my $file = $entry->file_path($self->cat());
	eval {
		my $cat_id = ($entry->cat == -1) ? '' : $entry->cat;
		my ($base,$css) = $self->_check_template('ent',$cat_id);
		my $info = (defined($param{'prev'}) or defined($param{'next'}))
		         ? {'neighbor'=>{'prev'=>$param{'prev'},'next'=>$param{'next'}}}
		         : undef;
		open(ENTRYOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(ENTRYOUT);
		print ENTRYOUT $content->output($base,
			'mode'      => 'ent',
			'css'       => $css,
			'id'        => $entry->id,
			'time'      => $self->time(),
			'blog'      => $self->blog(),
			'user'      => $self->user(),
			'cat'       => $self->cat(),
			'sortedcat' => $self->sortedcat(),
			'entryinfo' => $info,
			'entry'     => [$entry],
			'entry_num' => 1,
		);
		close(ENTRYOUT);
		chmod($self->conf->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
sub build_monthly_archive
{
	my $self = shift;
	my $month = shift;
	return sb->string('Unknown error.') if (!$month);
	my $file = $self->conf->value('conf_dir_base') 
	         . $self->conf->value('conf_dir_log')
	         . $month
	         . $self->conf->value('basic_suffix');
	my $content = sb::Content->new(
		'main' => {
			# 'category'     => \&_category_list,
			# 'archives'     => sub { _script_tag('archives_list',$_[0]); },
			# 'latest_entry' => sub { _script_tag('latest_entry_list',$_[0]); },
		}
	);
	eval {
		my ($base,$css) = $self->_check_template('arc',undef);
		open(MONTHLYOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(MONTHLYOUT);
		print MONTHLYOUT $content->output($base,
			'mode'      => 'arc',
			'css'       => $css,
			'cond'      => $month,
			'time'      => $self->time(),
			'blog'      => $self->blog(),
			'user'      => $self->user(),
			'cat'       => $self->cat(),
			'sortedcat' => $self->sortedcat(),
		);
		close(MONTHLYOUT);
		chmod($self->conf->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
sub build_category_index
{
	my $self = shift;
	my $cat_id = shift;
	return sb->string('Unknown error.') if ($cat_id eq '' or $cat_id == -1);
	my $cat = $self->cat()->{$cat_id};
	return sb->string('Unknown error.') if (!$cat);
	return if ($cat->dir eq '' or !$cat->idx); # it's not necessary to create the index.
	my $file = $self->conf->value('conf_dir_base') . $cat->dir . $self->conf->value('file_index');
	my $content = sb::Content->new(
		'main' => {
			'_rssfeed'     => \&_category_feeds,
			# 'category'     => \&_category_list,
			# 'archives'     => sub { _script_tag('archives_list',$_[0]); },
			# 'latest_entry' => sub { _script_tag('latest_entry_list',$_[0]); },
		}
	);
	eval {
		my ($base,$css) = $self->_check_template('arc',$cat->id);
		open(CATOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(CATOUT);
		print CATOUT $content->output($base,
			'mode'      => 'cat',
			'id'        => $cat->id,
			'css'       => $css,
			'time'      => $self->time(),
			'blog'      => $self->blog(),
			'user'      => $self->user(),
			'cat'       => $self->cat(),
			'sortedcat' => $self->sortedcat(),
			'entryinfo' => $self->entryinfo(),
		);
		close(CATOUT);
		chmod($self->conf->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
sub build_top_page
{
	my $self = shift;
	my $file = $self->conf->value('conf_dir_base') . $self->conf->value('file_index');
	my $content = sb::Content->new();
	eval {
		my ($base,$css) = $self->_check_template(undef,undef);
		open(TOPOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(TOPOUT);
		print TOPOUT $content->output($base,
			'mode'      => 'page',
			'css'       => $css,
			'time'      => $self->time(),
			'blog'      => $self->blog(),
			'user'      => $self->user(),
			'cat'       => $self->cat(),
			'sortedcat' => $self->sortedcat(),
			'entryinfo' => $self->entryinfo(),
			'entry'     => ( $self->conf->value('conf_entry_sort') ) ? $self->entry() : undef,
			'entry_num' => $self->entry_num(),
		);
		close(TOPOUT);
		chmod($self->conf->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
sub build_javascript
{
	my $self = shift;
	my $type = shift;
	my $dir = $self->conf->value('conf_dir_base') . $self->conf->value('conf_dir_log');
	my @types = keys( %mListScripts );
	if (grep { /\Q$type\E/ } @types)
	{
		@types = ($type);
	}
	else
	{
		$type = 'all';
	}
	my $content = sb::Content->new();
	foreach my $type ( @types )
	{
		my $block = $mListScripts{$type}->{'block'};
		my $file = $dir . $mListScripts{$type}->{'js'};
		my $cms  = sb::Template->new(sprintf(LIST_TEMPLATE,$block,$type,$block));
		my $temp = $content->output($cms,
			'time'      => $self->time(),
			'blog'      => $self->blog(),
			'user'      => $self->user(),
			'cat'       => $self->cat(),
			'sortedcat' => $self->sortedcat(),
			'entryinfo' => $self->entryinfo(),
			'entry'     => [],
			'entry_num' => 0,
		);
		$self->_generate_javascript($file,$temp);
	}
	return if ($type ne 'all' and $type ne 'category_list');
	# $type eq 'all' or $type eq 'category_list'
	foreach my $cat ( values %{$self->cat()} )
	{
		next if ($cat->sub eq '');
		my $file = $dir . SUBCATEGORY_PREFIX . $cat->id . '.js';
		my $cms  = sb::Template->new(sprintf(LIST_TEMPLATE,'category','subcategory_list','category'));
		my $temp = $content->output($cms,
			'mode'      => 'cat',
			'id'        => $cat->id,
			'time'      => $self->time(),
			'blog'      => $self->blog(),
			'user'      => $self->user(),
			'cat'       => $self->cat(),
			'sortedcat' => $self->sortedcat(),
			'entryinfo' => $self->entryinfo(),
			'entry'     => [],
			'entry_num' => 0,
		);
		$self->_generate_javascript($file,$temp);
	}
}
sub build_cookie_js
{
	my $self  = shift;
	my $force = shift;
	my $file = $self->conf->value('conf_dir_base') . $self->conf->value('conf_dir_log') . $self->conf->value('file_cook');
	return if (-s $file and !$force);
	my $temp  = sb::Template->new(sb::App->load_template('file'=>COOKIE_JS_TEMP));
	my @icons = sb::Data->load('Image','cond'=>{'icon_c'=>1});
	my $icon_list = '';
	foreach my $icon (@icons)
	{
		$icon_list .= "\t" . 'case \'' . $icon->id . '\': imgSrc = "';
		$icon_list .= $icon->get_url . '"; break;' . "\n";
	}
	$temp->num(0);
	$temp->tag('sb_cookie_tag'=>$self->conf->value('basic_cooktag'));
	$temp->tag('sb_icon_list'=>$icon_list);
	eval {
		open(COOKIEJS,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(COOKIEJS);
		print COOKIEJS $temp->output;
		close(COOKIEJS);
		chmod($self->conf->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
sub build_feedfile
{
	my $self = shift;
	my $type = shift;
	my @feeds = keys( %mFeedFile );
	if (grep { /\Q$type\E/ } @feeds)
	{
		@feeds = ($type);
	}
	else
	{
		$type = 'all';
	}
	my $content = sb::Content->new('main' => { sb::Content::Feed::_init() });
	foreach my $feed (@feeds)
	{
		my $num = $self->conf->value('basic_gen_' . $feed);
		my $filebase = $self->conf->value('conf_dir_log') . $mFeedFile{$feed};
		my $file = $self->conf->value('conf_dir_base') . $filebase;
		my $temp = sb::App->load_template('file'=>sb::App::Feed->template_name($feed));
		sb::Content::FeedParam->new->path($self->conf->value('conf_srv_base') . $filebase);
		$self->set_latest_entries($num) if ($num != $self->conf->value('conf_entry_disp'));
		next if ($temp eq '');
		next if ($num == 0);
		my $body = $content->output(sb::Template->new($temp),
			'mode'      => 'page',
			'time'      => $self->time(),
			'blog'      => $self->blog(),
			'user'      => $self->user(),
			'cat'       => $self->cat(),
			'sortedcat' => $self->sortedcat(),
			'entryinfo' => $self->entryinfo(),
			'entry'     => $self->entry(),
			'entry_num' => $self->entry_num(),
			'num'       => $num,
		);
		if ($self->lang->charcode ne sb::App::Feed->get_code())
		{ # convert character code
			$body = $self->lang->convert($body,sb::App::Feed->get_code(),$self->lang->charcode);
		}
		eval {
			open(TOPOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
			binmode(TOPOUT);
			print TOPOUT $body;
			close(TOPOUT);
			chmod($self->conf->value('basic_file_attr'),$file);
		};
		last if ($@);
	}
	return $@ if ($@);
	return;
}
# --------------------------------------
# // public functions - accessors
sub template
{
	my $self = shift;
	return $self->{'template'} if ($self->{'template'});
	my %template_of = (
		'css'  => undef,
		'html' => {},
	);
	my @templates = sb::Data->load('Template');
	foreach my $temp ( @templates )
	{
		next if ($temp->main eq '' and $temp->entry eq '');
		if ($temp->use)
		{
			$self->default_id($temp->id);
			$template_of{'css'} = sb::Template->new($temp->css) if ($temp->css ne '');
		}
		$template_of{'html'}->{$temp->id} = {
			'main'  => ($temp->main ne '')  ? sb::Template->new($temp->main)  : undef,
			'entry' => ($temp->entry ne '') ? sb::Template->new($temp->entry) : undef,
		};
	}
	die(sb->string('Unknown error.')) if ($self->default_id() eq '');
	$self->{'template'} = \%template_of;
	return $self->{'template'};
}
sub entryinfo
{
	my $self = shift;
	return $self->{'entryinfo'} if ($self->{'entryinfo'});
	$self->{'entryinfo'} = sb::Content::_entry_info(
		'time' => $self->time(),
		'mode' => 'page',
		'conf' => $self->conf,
	);
	return $self->{'entryinfo'};
}
sub set_latest_entries
{
	my $self = shift;
	my $num  = shift;
	$num = $self->conf->value('conf_entry_disp') if (!$num);
	$self->{'entry'} = [sb::Data->load('Entry',
			'num'   => $num,
			'cond'  => {'stat'=>1},
			'sort'  => 'date',
			'order' => 1,
		)
	];
	$self->{'entry_num'} = sb::Data->matched;
}
# --------------------------------------
# // private functions - utilities
sub _check_template
{
	my $self = shift;
	my $mode = shift; # mode 'ent','arc', or none
	my $cid  = shift; # category id
	my ($base,$css); # output parameters
	my $temp = undef;
	my $cat = ($cid ne '') ? $self->cat()->{$cid} : undef;
	my $tid = ($mode eq 'arc') ? $self->conf->value('conf_archive_temp') : -1;
	$tid = $cat->temp if ($cat and $cat->temp ne '' and $cat->temp > -1);
	$tid = undef if ($tid == -1);
	$css = ( $tid ne '' and $self->conf->value('conf_css_change') ) # css path
	     ? $self->conf->value('conf_srv_cgi') . $self->conf->value('basic_sb') . '?css=' . $tid 
	     : $self->conf->value('conf_srv_base') . $self->conf->value('file_css');
	$temp = $self->template()->{'html'}->{$tid} if ($tid ne '');
	$temp = $self->template()->{'html'}->{$self->default_id()} if (!$temp);
	$base = ($mode eq 'ent' and $temp->{'entry'}) # template base
	      ? $temp->{'entry'} 
	      : $temp->{'main'}; 
	$base->clear();
	return($base,$css);
}
sub _generate_javascript
{
	my $self = shift;
	my $file = shift;
	my $body = shift;
	eval {
		open(JSOUT,">$file") or die(sb->string('Failed to open the file') . ' : ' . $file);
		binmode(JSOUT);
		foreach my $line ( split("\n",$body) )
		{
			$line =~ s/\'/&#39;/g;
			print JSOUT 'document.write(\'' . $line . '\\n\');',"\n";
		}
		close(JSOUT);
		chmod($self->conf->value('basic_file_attr'),$file);
	};
	return $@ if ($@);
	return;
}
# --------------------------------------
# // private functions - content
sub _script_tag
{
	my $key = shift;
	my $cms = shift;
	my $conf = sb::Config->get;
	my $file = $conf->value('conf_dir_log') . $mListScripts{$key}->{'js'};
	return 0 if (!-e $conf->value('conf_dir_base') . $file);
	return 0 if (-z $conf->value('conf_dir_base') . $file);
	$cms->num(0);
	$cms->tag($key => sprintf(SCRIPT_TAG,sb::Language->get->charset,$conf->value('conf_srv_base') . $file));
	return 1;
}
sub _calendar_for_entry
{
	_script_tag('calendar',$_[0]);
	_script_tag('calendar2',$_[0]);
	_script_tag('calendar_vertical',$_[0]);
	_script_tag('calendar_horizontal',$_[0]);
	return 1;
}
sub _category_list
{
	my $cms = shift;
	my %var = @_;
	my $check = _script_tag('category_list',$cms);
	if ($check and $var{'entry'}->[0] and $var{'entry'}->[0]->cat ne '')
	{
		my $cat = $var{'cat'}->{$var{'entry'}->[0]->cat};
		if ($cat and $cat->sub ne '')
		{
			my $path = $var{'conf'}->value('conf_srv_base') . $var{'conf'}->value('conf_dir_log');
			$path .= SUBCATEGORY_PREFIX . $cat->id . '.js';
			$cms->num(0);
			$cms->tag('subcategory_list'=>sprintf(SCRIPT_TAG,$var{'lang'}->charset,$path));
		}
		return 1;
	}
	return $check;
}
sub _category_feeds
{
	my $cms = shift;
	my %var = @_;
	my $path = $var{'conf'}->value('conf_srv_cgi') . $var{'conf'}->value('basic_feed');
	$cms->tag('site_rss' =>$path . 'rss&amp;cid=' . $var{'id'});
	$cms->tag('site_atom'=>$path . 'atom&amp;cid=' . $var{'id'});
}
1;
__END__
