# sb::Content::Mobile
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Content::Mobile;
use vars qw( $VERSION );
$VERSION = '0.00';
# 0.00 [2007/08/17] separated from sb::App::Mobile
# --------------------------------------
# // defines / static variables
sub OUTPUT_CHARSET    (){ 'Shift_JIS' };
sub DEFAULT_TEXT_SIZE (){ 2048 };
sub MOBILE_TAGS       (){ 'BR|A|Q|IMG|EM|STRONG|H\d|P|DIV|[O|U|D]L|LI|DT|DD|BLOCKQUOTE|FORM|INPUT|SELECT|OPTION|HR|DEL|PRE' };
sub ENTRY_LINK        (){ '<a href="%s" accesskey="%s">[%s] %s</a> ' };
sub COMMENTFORM_LINK  (){ 'comment form' };
sub BODY_LINK         (){ 'beginning...' };
sub MORE_LINK         (){ 'more...' };
sub PREV_ARROW        (){ '&lt;&lt;' };
sub NEXT_ARROW        (){ '&gt;&gt;' };
# --------------------------------------
# // default callbacks
sub _init
{
	return (
		'_mobile'               => \&_main,
		'mobile_xmldeclaration' => \&_xmldeclaration,
		'mobile_top'            => \&_top,
		'mobile_entry'          => \&_entry,
		'mobile_comment_area'   => \&_comment_area,
		'mobile_comment_form'   => \&_comment_form,
		'mobile_trackback_area' => \&_trackback_area,
	);
}
# --------------------------------------
# // define callbacks
sub _main
{
	my $cms = shift;
	my %var = @_;
	$cms->tag('site_encoding'=>OUTPUT_CHARSET);
	return(1);
}
sub _xmldeclaration
{
	my $cms = shift;
	my %var = @_;
	my $param = sb::Content::MobileParam->get();
	return ($param->agent() =~ /DoCoMo/) ? 0 : 1;
}
sub _top
{
	my $cms = shift;
	my %var = @_;
	my $param = sb::Content::MobileParam->get();
	return 0 if ( $param->mode() ne 'page' and $param->mode() ne 'cat' );
	sb::Content::Common::_title($cms,%var);
	sb::Content::List::_selected($cms,%var);
	$var{'id'} = sb::Interface->get->value('cid') if ($param->mode() eq 'cat'); # set parameter temporarily
	sb::Content::Common::_page($cms,%var);
	return 1;
}
sub _entry
{
	my $cms   = shift;
	my %var   = @_;
	my $entry = $var{'entry'}->[0];
	my $param = sb::Content::MobileParam->get();
	return 0 if ( $param->mode() ne 'ent' );
	return 0 if ( !$entry );
	my $funcs = { sb::Content::Entry::_init() };
	$funcs->{'body_text'} = \&_entry_body;
	foreach my $label (keys %{ $funcs } )
	{
		next if ($label eq '_main');
		eval{ &{$funcs->{$label}}($cms,$entry,%var) };
	}
	return 1;
}
sub _entry_body
{
	my $cms   = shift;
	my $entry = shift;
	my %var   = @_;
	my $param = sb::Content::MobileParam->get();
	my $permalink = $entry->permalink('type'=>'Mobile');
	my $nxt  = $param->page();
	my $body = (! $param->more() ) 
		? $entry->formatted_body() 
		: $entry->formatted_more();
	$body = sb::Text->new($body)->tag_removed(
		'code'  => $var{'lang'}->charcode,
		'allow' => MOBILE_TAGS,
	);
	$body = _change_tags($body);
	($body,$nxt) = _paging_entry($body,$param->page()) if (length($body) > DEFAULT_TEXT_SIZE);
	$cms->tag('entry_description'=>$body);
	$cms->tag('entry_excerpt'=>$entry->sum);
	# entry links
	my $mark = (! $param->more() ) ? '&amp;page=' : '&amp;more=';
	my $navi = '';
	if ($nxt > $param->page())
	{
		$navi .= sprintf(ENTRY_LINK,$permalink . $mark . ($param->page() - 1),'7','7',PREV_ARROW) if ($param->page() > 0);
		$navi .= sprintf(ENTRY_LINK,$permalink,'7','7',BODY_LINK) if ($param->page() == 0 and $param->more());
		$navi .= sprintf(ENTRY_LINK,$permalink . $mark . $nxt,'9','9',NEXT_ARROW);
	}
	else
	{
		$navi .= sprintf(ENTRY_LINK,$permalink . $mark . ($param->page() - 1),'7','7',PREV_ARROW) if ($param->page() > 0);
		if (!$param->more() and $entry->more ne '')
		{
			$navi .= sprintf(ENTRY_LINK,$entry->permalink('type'=>'Mobile','mode'=>'more'),'9','9',MORE_LINK);
		}
		elsif ($param->more() and $param->page() == 0)
		{
			$navi .= sprintf(ENTRY_LINK,$permalink,'7','7',BODY_LINK);
		}
	}
	$cms->tag('entry_link'=>$navi);
}
sub _comment_area
{
	my $cms = shift;
	my %var = @_;
	my $param = sb::Content::MobileParam->get();
	my $entry = $var{'entry'}[0];
	return 0 if ( $param->mode() ne 'msg' );
	return 0 if (!$entry);
	return 0 if ($entry->acm == 0);
	my @comments = _paging_attachment(
		sb::Data->load('Message',
			'sort'   => 'date',
			'cond'   => {'stat'=>1,'eid'=>$var{'id'}},
			'order'  => $var{'conf'}->value('conf_com_sort'),
		)
	);
	for (my $i=0;$i<@{$comments[$param->page()]};$i++)
	{
		my $com = $comments[$param->page()]->[$i];
		$cms->num($i);
		sb::Content::Message::_content($cms,$com,%var);
		my $body = $com->formatted_body;
		$body =~ s/&amp;(#\d*?;)/&$1/g; # displaying pictorial letters
		$cms->tag('comment_description'=>$body);
	}
	$cms->block('comment'=>($#{$comments[$param->page()]} + 1));
	$cms->num(0);
	$cms->tag('entry_link'=>sprintf(ENTRY_LINK,$entry->permalink('type'=>'Mobile'),'2','2',$entry->subj));
	$cms->tag('comment_form'=>
		sprintf(ENTRY_LINK,$entry->permalink('type'=>'Mobile','mode'=>'form'),'5','5',COMMENTFORM_LINK)
	);
	my $permalink = $entry->permalink('type'=>'Mobile');
	my $navi = '';
	my $mark = '&amp;com=';
	$navi .= sprintf(ENTRY_LINK,$permalink . $mark . ($param->page() - 1),'7','7',PREV_ARROW) if ($param->page() > 0);
	$navi .= sprintf(ENTRY_LINK,$permalink . $mark . ($param->page() + 1),'9','9',NEXT_ARROW) if ($param->page() < $#comments);
	$cms->tag('comment_link'=>$navi);
	return(1);
}
sub _comment_form
{
	my $cms = shift;
	my %var = @_;
	my $param = sb::Content::MobileParam->get();
	my $entry = $var{'entry'}[0];
	return(0) if ( $param->mode() ne 'form' );
	return(0) if (!$entry);
	return(0) if ($entry->acm == 0);
	$cms->num(0);
	$cms->tag('comment_num'=>
		sprintf(ENTRY_LINK,
			$entry->permalink('type'=>'Mobile','mode'=>'com'),
			'5',
			'5',
			$var{'lang'}->string('parts_com_num') . ' (' . $entry->com . ')'
		)
	);
	$cms->tag('entry_link'=>sprintf(ENTRY_LINK,$entry->permalink('type'=>'Mobile'),'2','2',$entry->subj));
	$cms->tag('entry_id'=>$entry->id);
	return(1);
}
sub _trackback_area
{
	my $cms = shift;
	my %var = @_;
	my $param = sb::Content::MobileParam->get();
	my $entry = $var{'entry'}[0];
	return(0) if ( $param->mode() ne 'tb' );
	return(0) if (!$entry);
	return(0) if ($entry->atb == 0);
	my @trackbacks = _paging_attachment(
		sb::Data->load('Trackback',
			'sort'   => 'date',
			'cond'   => {'stat'=>1,'eid'=>$var{'id'}},
			'order'  => $var{'conf'}->value('conf_tb_sort'),
			'detail' => 'on',
		)
	);
	for (my $i=0;$i<@{$trackbacks[$param->page()]};$i++) {
		my $tb = $trackbacks[$param->page()]->[$i];
		$cms->num($i);
		sb::Content::Trackback::_content($cms,$tb,%var);
	}
	$cms->block('trackback'=>($#{$trackbacks[$param->page()]} + 1));
	$cms->num(0);
	$cms->tag('entry_link'=>sprintf(ENTRY_LINK,$entry->permalink('type'=>'Mobile'),'2','2',$entry->subj));
	my $permalink = $entry->permalink('type'=>'Mobile');
	my $navi = '';
	my $mark = '&amp;tb=';
	$navi .= sprintf(ENTRY_LINK,$permalink . $mark . ($param->page() - 1),'7','7',PREV_ARROW) if ($param->page() > 0);
	$navi .= sprintf(ENTRY_LINK,$permalink . $mark . ($param->page() + 1),'9','9',NEXT_ARROW) if ($param->page() < $#trackbacks);
	$cms->tag('trackback_link'=>$navi);
	return(1);
}
# --------------------------------------
# // utilities
sub _paging_entry
{
	my ($text,$page) = @_; # input parameters / $page works as output parameter as well
	my $output = '';       # output parameter
	my @lines = split("\n",$text);
	my @tags = ();
	my $cnt  = 0;
	my $size = 0;
	my $bgn  = $page * DEFAULT_TEXT_SIZE;
	my $end  = ($page + 1) * DEFAULT_TEXT_SIZE;
	foreach my $line (@lines)
	{
		my @check = ($line =~ /<(.*?)>/g); # pick up all tags in the line
		if ($size >= $bgn and $size < $end)
		{
			if ($output eq '' and $#tags > -1)
			{
				foreach (@tags)
				{
					$output .= '<' . $_ . '>';
				}
			}
			$output .= $line . "\n";
		}
		foreach my $tag (@check)
		{
			next if (index($tag,'\!') == 0 or index($tag,'\?') == 0);
			next if (index($tag,'/') > 0);
			next if ($tag =~ /^(a|br|hr)/i); # empty elements and anchors
			if (index($tag,'/') == -1)
			{
				$tag = lc((split(/\s/,$tag))[0]);
				push(@tags,$tag);
				next;
			}
			elsif (index($tag,'/') == 0)
			{
				$tag =~ tr/\///d;
				$tag = lc($tag);
				pop(@tags) if ($tags[$#tags] eq $tag);
			}
		}
		$size += length($line);
		if ($size > $end)
		{
			if ($#tags > -1)
			{
				@tags = reverse(@tags);
				foreach (@tags)
				{
					$output .= '</' . $_ . '>';
				}
			}
			last;
		}
		$cnt++;
	}
	$output = $lines[$cnt] if ($output eq '');
	$page++ if ($cnt < $#lines);
	return( $output,$page );
}
sub _change_tags
{
	my $text = shift;
	$text =~ s/<a(.*?)href\s?=\s?"(.*?)"((?:(?!<\/a>))*?)><img (.*?)alt\s?=\s?"(.*?)"(.*?)><\/a>/<a href="$2">image[$5]<\/a>/sgi; # by trip_eye
	$text =~ s/<img (.*?)src\s?=\s?"(.*?)"(.*?)alt\s?=\s?"(.*?)"(.*?)>/<a href="$2">image[$4]<\/a>/sgi;
	$text =~ s/<img (.*?)alt\s?=\s?"(.*?)"(.*?)src\s?=\s?"(.*?)"(.*?)>/<a href="$4">image[$2]<\/a>/sgi;
	$text =~ s/<img (.*?)src\s?=\s?"(.*?)"(.*?)>/<a href="$2">image<\/a>/sgi;
	$text =~ s/<\/?q(.*?)>/&quot;/sgi;
	$text =~ s/<del(.*?)<\/del>//sgi;
	return $text;
}
sub _paging_attachment
{
	my @objs = @_;
	my @page = ([]);
	my $size = 0;
	my $cnt  = 0;
	for (my $i=0;$i<@objs;$i++)
	{
		$size += $objs[$i]->get_size;
		push(@{$page[$cnt]},$objs[$i]);
		if ($size >= DEFAULT_TEXT_SIZE)
		{
			$cnt++;
			$size = 0;
			$page[$cnt] = [];
		}
	}
	return @page;
}
# ======================================
package sb::Content::MobileParam;
use base qw( sb::RequestObject );
sub initialize
{
	my $self = shift;
	my %param = (
		'page'  => 0,
		'num'   => 0,
		'mode'  => '',
		'agent' => '',
		'more'  => undef,
	);
	$self->SUPER::initialize(%param);
	$self->generate_accessor( keys(%param) );
	return $self;
}
1;
__END__
