# sb::Plugin::Memo
# == written by T.Otani <takuya.otani@gmail.com> ===
# == Copyright (C) 2008 SimpleBoxes/SerendipityNZ ==

use strict;

if ( !defined $sb::VERSION
  or $sb::VERSION =~ /^2.(\d\d)[BRD]$/i )
{
	die("This plugin works under Serene Bach 3.* only.\n");
}

# ======================================
package sb::Plugin::Memo;
use sb::Plugin ();
use sb::Text ();
use sb::Time ();
use sb::Build ();
use sb::Admin::Profile ();
use base qw( sb::Admin::Profile );
# --------------------------------------
# // defines
sub TEMPLATE (){ 'memo.html' };
# --------------------------------------
# // plugin registration
# register plugin
sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'utf8',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'admin, cms',
		'name'    => 'Short Message' . _getname(),
		'text'    => 'Writing short message on the top page.',
		'author'  => 'takkyun',
		'detail'  => 'http://serenebach.net/',
		'version' => version(),
	},
	'file' => 'memo.txt',
	'data' => 1,
);
# register as admin module
sb::Plugin->register_admin_module(
	'mode'   => 'memo' . _getname(),
	'level'  => 1,
	'module' => __PACKAGE__,
);
# register as content module
sb::Plugin->register_content_module(
	'type'     => 'main',
	'callback' => \&_topmemo,
	'field'    => 'topmemo' . _getname(),
);
_initialize_string(); # initialize string
# --------------------------------------
# // common functions
sub version
{
	return '0.05';
	# 0.05 [2008/11/24] fixed typo
	# 0.04 [2008/11/23] fixed degradation / bug in _topmemo
	# 0.03 [2008/11/18] ported for Serene Bach 3.*
	# 0.02 [2008/03/06] modified structure to consider package name
	# 0.01 [2006/08/03] changed detail to point new site address
}
sub _getname
{
	my $name = __PACKAGE__;
	return $1 if ($name =~ /(\d+$)/);
	return;
}
sub _initialize_string
{
	my $string = sb::Language->get->string('mode_memo' . _getname());
	$string .= _getname();
	sb::Language->get->string('mode_memo' . _getname() => $string);
}
# --------------------------------------
# // functions for content
sub _topmemo
{
	my $cms = shift;
	my %var = @_;
	my $data = sb::Plugin->get_data;
	return 0 if ($var{'mode'} ne 'page' or $var{'page'} != 0 or !$data); # not top page
	my $date = sb::Time->format(
		'time'=>$data->date,
		'form'=>$var{'conf'}->value('conf_entry_date') . ' ' . $var{'conf'}->value('conf_entry_time'),
		'zone'=>$var{'conf'}->value('conf_timezone'),
		'lang'=>$var{'conf'}->value('conf_time_lang')
	);
	my $memo = ($data->setting ne 'on') 
	         ? sb::Text->new($data->data)->format(1) 
	         : $data->data;
	$cms->num(0);
	$cms->tag('sb_memo' . _getname() => $memo);
	$cms->tag('sb_memo' . _getname() . '_time' => $date);
	return 1;
}
# --------------------------------------
# // functions for admin
sub callback
{
	my $self = shift;
	return ( $self->regi() )
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
sub _update
{
	my $self = shift;
	my $data = sb::Plugin->get_data;
	$data->data($self->cgi->value('topmemo'));
	$data->setting($self->cgi->value('topmemo_breaks'));
	$data->date($self->time->data);
	sb::Plugin->set_data('data'=>$data);
	my %cat = sb::Data->load_as_hash('Category');
	my $builder = sb::Build->new(
		'time'      => $self->time->data,
		'user'      => $self->users,
		'cat'       => \%cat,
		'sortedcat' => [ sort { $b->order <=> $a->order } values(%cat) ],
		'blog'      => sb::Data->load('Weblog','id'=>0),
	);
	$builder->build_top_page();
	return $self->_display('message'=>sb->string('Modified.<br />'));
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		@_
	);
	my $temp = sb::Template->new(sb::Plugin->load_template('file'=>TEMPLATE));
	my $data = sb::Plugin->get_data;
	my $editor = $self->get_editor()->new('user' => $self->user);
	$temp->num(0);
	$temp->tag('sb_topmemo_breaks' => ($data->setting ne '') ? 'checked="checked"' : '');
	$temp->tag('sb_topmemo' => $editor->display_editor(
		'id' => 'topmemo',
		'content' => sb::Text->entitize($data->data),
	));
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
1;
__END__
